/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "base.h"

#include <cstring>

using namespace std;

#ifndef HAVE_EXP10_SET
ieee exp10(ieee x){
  return exp(LOG10*x);
}
#endif

value::value(uint t){
  type=t;
}

itype value::assertI(const char* op){
  if(type!=INTEGER){
    fprintf(stderr,"warning: integer required by '%s', converting\n",op);
  }
  return i;
}

ieee value::assertD(const char* op){
  if(type!=FLOATING){
    fprintf(stderr,"warning: floating required by '%s', converting\n",op);
  }
  return d;
}

value value::operator+(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return value::integer(i+v.i);
  }else{
    return value::floating(d+v.d);
  }
}

value value::operator-(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return value::integer(i-v.i);
  }else{
    return value::floating(d-v.d);
  }
}

value value::operator*(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return value::integer(i*v.i);
  }else{
    return value::floating(d*v.d);
  }
}

value value::operator/(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    if(v.i==0)return value::integer(0);
    return value::integer(i/v.i);
  }else{
    if(v.d==0)return value::integer(0);
    return value::floating(d/v.d);
  }
}

value value::intpow(itype i1, itype i2){
  if(i2<0){
    return value::floating(pow((ieee)i1,(int)i2));
  }else{
    int sign=(i1<0) && (i2%2==1)?-1:1;
    i1=i1<0?-i1:i1;
    itype trailing=i1;
    itype total=1;
    while(i2!=0){
      if(i2&1)total*=trailing;
      trailing*=trailing;
      i2>>=1;
    }
    return value::integer(total*sign);
  }
}
value value::flpow(ieee i1, itype i2){
  if(i2<0){
    return value::floating(pow(i1,(int)i2));
  }else{
    int sign=(i1<0) && (i2%2==1)?-1:1;
    i1=i1<0?-i1:i1;
    ieee trailing=i1;
    ieee total=1;
    while(i2!=0){
      if(i2&1)total*=trailing;
      trailing*=trailing;
      i2>>=1;
    }
    return value::floating(total*sign);
  }
}

value value::operator^(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return intpow(i,v.i);
  }else if(v.type==INTEGER){
    return flpow(d,v.i);
  }else{
    return value::floating(pow(d,v.d));
  }
}

void value::fractionalize(ieee d, itype& p, ieee& q){
     if(d>=0){
         p=(itype)d;
         q=d-p;
         return;
     }
     p=(itype)(d-(itype)d+1)+(itype)d-1;
     //p=-(int)(-d)-1;
     q=d-p;
}

ieee value::dmod(ieee d1, ieee d2){
    ieee r=d1-(floor(d1/d2))*d2;
    return r<0?0:r;
}

ieee value::f_mod(ieee v1, ieee v2){
  if(v2==0)return v1;
  int sign=v2<0?-1:1;
  v2*=sign;
  if(sign==1){
    return dmod(v1,v2);
  }else{
    return dmod(v1,v2)-v2;
  }
}

itype value::i_mod(itype i1, itype i2){
  if(i2==0)return i1;
  int sign=i2<0?-1:1;
  i2*=sign;
  if(sign==1){
    return i1<0?(i2-(-i1)%i2)%i2:i1%i2;
  }else{
    return (i1<0?(i2-(-i1)%i2)%i2:i1%i2)-i2;
  }
}

value value::operator%(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return value::integer(i_mod(i,v.i));
  }else{
    return value::floating(f_mod(d,v.d));
  }
}



value value::operator+(const ieee& v){
  return value::floating(d+v);
}

value value::operator-(const ieee& v){
  return value::floating(d-v);
}

value value::operator*(const ieee& v){
  return value::floating(d*v);
}

value value::operator/(const ieee& v){
  if(v==0)return value::integer(0);
  return value::floating(d/v);
}

value value::operator^(const ieee& v){
  return value::floating(pow(d,v));
}

value value::operator%(const ieee& v){
  return value::floating(f_mod(d,v));
}




value value::operator+(const itype& v){
  if(type==INTEGER){
    return value::integer(i+v);
  }else{
    return value::floating(d+v);
  }
}

value value::operator-(const itype& v){
  if(type==INTEGER){
    return value::integer(i-v);
  }else{
    return value::floating(d-v);
  }
}

value value::operator*(const itype& v){
  if(type==INTEGER){
    return value::integer(i*v);
  }else{
    return value::floating(d*v);
  }
}

value value::operator/(const itype& v){
  if(v==0)return value::integer(0);
  if(type==INTEGER){
    return value::integer(i/v);
  }else{
    return value::floating(d/v);
  }
}

value value::operator^(const itype& v){
  if(type==INTEGER){
    return intpow(i,v);
  }else {
    return flpow(d,v);
  }
}

value value::operator%(const itype& v){
  if(type==INTEGER){
    return value::integer(i_mod(i,v));
  }else{
    return value::floating(f_mod(d,v));
  }
}


value value::operator-(){
  if(type==INTEGER){
    return value::integer(-i);
  }else{
    return value::floating(-d);
  }
}

value value::op_sin(){
  return value::floating(sin(d));
}

value value::op_cos(){
  return value::floating(cos(d));
}

value value::op_tan(){
  return value::floating(tan(d));
}

value value::op_atan(){
  return value::floating(atan(d));
}

value value::op_asin(){
  return value::floating(asin(d));
}

value value::op_acos(){
  return value::floating(acos(d));
}

value value::op_abs(){
  if(type==INTEGER){
    return value::integer(i<0?-i:i);
  }else{
    return value::floating(fabs(d));
  }
}

value value::op_exp(){
  return value::floating(exp(d));
}

value value::op_sqrt(){
  return value::floating(sqrt(d));
}

value value::op_cbrt(){
  return value::floating(cbrt(d));
}

value value::op_log(){
  return value::floating(log(d));
}


value value::op_bj(int n){
  return value::floating(jn(n,d));
}

value value::op_by(int n){
  return value::floating(yn(n,d));
}

value value::cast_int(){
  return value::integer(i);
}

value value::cast_float(){
  return value::floating(d);
}

value value::op_frac(){
  return value::floating(f);
}

value value::op_round(){
  return value::integer(f>=0.5?i+1:i);
}

value value::op_fact(){
  if(type==INTEGER){
#if IVALUE==ILONG
  if(i>20)
#else
  if(i>12)
#endif
  {
    ieee fac=1;
    for(itype u=2;u<=i;u++)fac*=u;
    return value::floating(fac);
  }else{
    itype fac=1;
    for(itype u=2;u<=i;u++)fac*=u;
    return value::integer(fac);
  }
  }else{
    int gi;
    ieee lg=lgamma_r(d+1,&gi);
    return value::floating(gi*exp(lg));
  }
}

value value::op_exp10(){
  return value::floating(exp10(d));
}

value value::op_rand(){
  if(type==INTEGER){
    if(!i)return value::integer(0);
    int sign=i<0?-1:1;
    return value::integer(sign*(rand()%(sign*i)));
  }else{
    return value::floating((d*rand())/RAND_MAX);
  }
}

value value::integer(itype i){
  value v(INTEGER);
  v.i=i;
  v.d=i;
  v.f=0;
  return v;
}

value value::floating(ieee f){
  value v(FLOATING);
  v.d=f;
  fractionalize(f,v.i,v.f);
  return v;
}

value value::binomial(value v1, value v2){
  if(v2.type==INTEGER){
    value total=integer(1);
    value nom=v1;
    for(int i=1;i<=v2.i;i++){
      total=total*v1;
      total=total/(itype)i;
      v1=v1-(itype)1;
    }
    return total;
  }else{
    int i1, i2, i3;
    ieee g1=lgamma_r(v1.d+1,&i1);
    ieee g2=lgamma_r(v2.d+1,&i2);
    ieee g3=lgamma_r(v1.d-v2.d+1,&i3);
    return value::floating((i1*i2*i3)*exp(g1-g2-g3));
  }
}

int value::is_odd(){
  if(type!=INTEGER)return 0;
  if(i<0){
    return (-i)%2==1;
  }else{
    return i%2==1;
  }
}

int value::is_even(){
  if(type!=INTEGER)return 0;
  if(i<0){
    return (-i)%2==0;
  }else{
    return i%2==0;
  }
}

int value::is_positive(){
  if(type==INTEGER)return i>=0;
  else return d>=0;
}

int value::is_negative(){
  if(type==INTEGER)return i<0;
  else return d<0;
}

int value::is_int(){
  return type==INTEGER;
}

int value::is_float(){
  return type==FLOATING;
}

int value::operator<(const value& v){
  return d<v.d;
}

int value::operator>(const value& v){
  return d>v.d;
}

int value::operator<=(const value& v){
  return d<=v.d;
}

int value::operator>=(const value& v){
  return d>=v.d;
}

int value::operator==(const value& v){
  if(type!=v.type)return 0;
  if(type==INTEGER)return i==v.i;
  else return d==v.d;
}

int value::operator!=(const value& v){
  if(type!=v.type)return 1;
  if(type==INTEGER)return i!=v.i;
  else return d!=v.d;
}

void value::print(FILE* f, char* Fd, char* Ff){
  switch(type){
    case INTEGER:
      fprintf(f,Fd,i);
      return;
    case FLOATING:
      fprintf(f,Ff,d);
      return;
  }
}

opid::opid(char* n, uchar c, uchar p, uchar a){
  name=n;
  code=c;
  precedence=p;
  asoc=a;
}

opid::opid(){
  name=0;
  code=OP_NONE;
}

//definition of shared::shared_ops, declared externally in the header
vector<opid> shared::shared_ops;
#ifdef SIGHANDLE
sig_atomic_t shared::interactive_flag=0;
sig_atomic_t shared::state=STATE_IDLE;
sig_atomic_t shared::interrupt=0; //volatile?
char* shared::wait_ptr=0;
#endif

void fill_ops(){
  //parentheses operators
  shared::shared_ops.push_back(opid(strdup("{}"),OP_LIST     ,0,ASOC_MULTI));
  shared::shared_ops.push_back(opid(strdup("[]"),OP_ITER1    ,0,ASOC_MULTI));
  shared::shared_ops.push_back(opid(strdup("()"),OP_ITER2    ,0,ASOC_MULTI));
  //special case
  shared::shared_ops.push_back(opid(strdup("-"),OP_MINUSSIGN ,11,ASOC_RUNARY));

  shared::shared_ops.push_back(opid(strdup("$"),OP_TAKE      ,9,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("@"),OP_PUT       ,9,ASOC_RBINARY));

  shared::shared_ops.push_back(opid(strdup(":+"),OP_PPLUS    ,4,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":-"),OP_MMINUS   ,4,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":*"),OP_MMUL     ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":/"),OP_DDIV     ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":%"),OP_MMOD     ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":^"),OP_PPOW     ,6,ASOC_RINFIX));

  shared::shared_ops.push_back(opid(strdup("@+"),OP_OPLUS    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("@*"),OP_OTIMES   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("avg"),OP_OMEAN   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("var"),OP_OVAR    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("nrm"),OP_ONORM   ,8,ASOC_RBINARY));

  shared::shared_ops.push_back(opid(strdup("!"),OP_FACT      ,9,ASOC_LUNARY));
  shared::shared_ops.push_back(opid(strdup("sin"),OP_SIN     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("cos"),OP_COS     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("tan"),OP_TAN     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("atn"),OP_ATAN    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("atan"),OP_ATAN   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("asn"),OP_ASIN    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("asin"),OP_ASIN   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("acs"),OP_ACOS    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("acos"),OP_ACOS   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("abs"),OP_ABS     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("exp"),OP_EXP     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("sqt"),OP_SQRT    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("sqrt"),OP_SQRT   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("cbt"),OP_CBRT    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("cbrt"),OP_CBRT   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("r"),OP_ROUND     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("f"),OP_FRAC      ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("i"),OP_INT       ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("d"),OP_FLOAT     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("log"),OP_LOG     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("ran"),OP_RAND    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("bj"),OP_BESJ     ,8,ASOC_RBINARY));
  shared::shared_ops.push_back(opid(strdup("by"),OP_BESY     ,8,ASOC_RBINARY));

  shared::shared_ops.push_back(opid(strdup("+"),OP_PLUS      ,4,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("-"),OP_MINUS     ,4,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("*"),OP_MUL       ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("/"),OP_DIV       ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("%"),OP_MOD       ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("^"),OP_POW       ,6,ASOC_RINFIX));
  shared::shared_ops.push_back(opid(strdup("e"),OP_FLE       ,10,ASOC_RINFIX));
  shared::shared_ops.push_back(opid(strdup("c"),OP_NCK       ,7,ASOC_RINFIX));

  shared::shared_ops.push_back(opid(strdup("&"),OP_AND       ,3,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("\\"),OP_WITHOUT  ,3,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("|"),OP_OR        ,2,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("|&"),OP_XOR      ,3,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("~"),OP_REVERSE   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("S"),OP_SORT      ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("#"),OP_HASH      ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("?"),OP_MASK      ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("="),OP_EQUALS    ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("#_"),OP_COUNT    ,8,ASOC_RBINARY));

  shared::shared_ops.push_back(opid(strdup("<<"),OP_SHL      ,7,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(">>"),OP_SHR      ,7,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("_"),OP_INDEX     ,7,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("U"),OP_UNIQUE    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("P"),OP_POSITIVE  ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("N"),OP_NEGATIVE  ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("O"),OP_ODD       ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("E"),OP_EVEN      ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("I"),OP_INTEGER   ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("D"),OP_DOUBLE    ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("min"),OP_MIN     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("max"),OP_MAX     ,8,ASOC_RUNARY));

  shared::shared_ops.push_back(opid(strdup(":<"),OP_LLT      ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":>"),OP_GGT      ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":<="),OP_LLEQ    ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":>="),OP_GGEQ    ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":=="),OP_EEQ     ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(":<>"),OP_NNEQ    ,1,ASOC_LINFIX));

  shared::shared_ops.push_back(opid(strdup("<"),OP_LT        ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(">"),OP_GT        ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("<="),OP_LEQ      ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup(">="),OP_GEQ      ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("=="),OP_EQ       ,1,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("<>"),OP_NEQ      ,1,ASOC_LINFIX));

  shared::shared_ops.push_back(opid(strdup("pi"),OP_PI       ,11,ASOC_BLANK));
  shared::shared_ops.push_back(opid(strdup("aa"),OP_AA       ,11,ASOC_BLANK));
  shared::shared_ops.push_back(opid(strdup("deg"),OP_DEG     ,11,ASOC_BLANK));
  shared::shared_ops.push_back(opid(strdup("emc"),OP_EMC     ,11,ASOC_BLANK));

  shared::shared_ops.push_back(opid(strdup("pm"),OP_PM       ,11,ASOC_BLANK));
  shared::shared_ops.push_back(opid(strdup("mp"),OP_MP       ,11,ASOC_BLANK));

  shared::shared_ops.push_back(opid(strdup("++"),OP_XPLUS    ,4,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("--"),OP_XMINUS   ,4,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("**"),OP_XMUL     ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("//"),OP_XDIV     ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("%%"),OP_XMOD     ,5,ASOC_LINFIX));
  //shared::shared_ops.push_back(opid(strdup("^^"),OP_XPOW     ,6,ASOC_RINFIX));
  shared::shared_ops.push_back(opid(strdup("px"),OP_PX       ,8,ASOC_RBINARY));
  shared::shared_ops.push_back(opid(strdup("pd"),OP_PD       ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("pz"),OP_PZ       ,8,ASOC_RUNARY));

  shared::shared_ops.push_back(opid(strdup("@:"),OP_DOT      ,5,ASOC_LINFIX));
  shared::shared_ops.push_back(opid(strdup("@="),OP_VLEN     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("@!"),OP_VNRM     ,8,ASOC_RUNARY));
  shared::shared_ops.push_back(opid(strdup("@^"),OP_CROSS    ,6,ASOC_RINFIX));
}

void kill_ops(){
  for(uint i=0;i<shared::shared_ops.size();i++){
    free(shared::shared_ops[i].name);
  }
}


opid findop(char* c){
  int idx=-1;
  int maxlen=0;
  int len;
  for(int i=4;i<shared::shared_ops.size();i++){
    len=strlen(shared::shared_ops[i].name);
    if(len<=maxlen)continue;
    if(!strncmp(shared::shared_ops[i].name,c,len)){
      maxlen=len;
      idx=i;
    }
  }
  if(idx<0)return opid(0,0,0,0);
  return shared::shared_ops[idx];
}
