/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#ifndef H_BASE
#define H_BASE

#include "config.h"

#define DEBUG 0
#define SIGHANDLE
//#define TIMERAND

#include <cmath>
#include <cstdlib>
#include <cstdio>

#include <vector>

#ifdef SIGHANDLE
#include <csignal>
#endif

using namespace std;

#define FLOATING 1
#define INTEGER  2

#define EPS 1e-32

//declare macros for different sizes of float and int values
#define FLOAT   4
#define DOUBLE  8
#define LONG   12

#define IINT   4
#define ILONG  8

//in case it wasn't declared by configure script
#ifndef VALUE
#define VALUE FLOAT
#endif

#ifndef IVALUE
#define IVALUE IINT
#endif



#if VALUE==FLOAT
typedef float ieee;
#define sqrt sqrtf
#define exp expf
#define log logf
#define sin sinf
#define cos cosf
#define tan tanf
#define sinh sinhf
#define cosh coshf
#define asin asinf
#define acos acosf
#define atan atanf
#define fabs fabsf
#define log10 log10f
#define ceil ceilf
#define floor floorf
#define pow powf
#define atan2 atan2f
#define cbrt cbrtf
#define jn jnf
#define yn ynf
#define lgamma lgammaf
#define lgamma_r lgammaf_r
#define erf erff
#define exp10 exp10f
#elif VALUE==DOUBLE
typedef double ieee;
#elif VALUE==LONG
typedef long double ieee;
#define sqrt sqrtl
#define exp expl
#define log logl
#define sin sinl
#define cos cosl
#define tan tanl
#define sinh sinhl
#define cosh coshl
#define asin asinl
#define acos acosl
#define atan atanl
#define fabs fabsl
#define log10 log10l
#define ceil ceill
#define floor floorl
#define pow powl
#define atan2 atan2l
#define cbrt cbrtl
#define jn jnl
#define yn ynl
#define lgamma lgammal
#define lgamma_r lgammal_r
#define erf erfl
#define exp10 exp10l
#endif

//in case exp10-like functions are not defined
#ifndef HAVE_EXP10_SET
#define LOG10 2.302585092994045684
ieee exp10(ieee x);
#endif

typedef unsigned char uchar;
typedef unsigned int uint;


#if IVALUE==IINT
typedef int itype;
#elif IVALUE==ILONG
typedef long long itype;
#endif

struct value {
  uint type;
  itype i;
  ieee f;
  ieee d;
  itype assertI(const char* op);
  ieee  assertD(const char* op);
  value operator+(const value& v);
  value operator-(const value& v);
  value operator*(const value& v);
  value operator/(const value& v);
  value operator^(const value& v);
  value operator%(const value& v);
  value operator+(const ieee& v);
  value operator-(const ieee& v);
  value operator*(const ieee& v);
  value operator/(const ieee& v);
  value operator^(const ieee& v);
  value operator%(const ieee& v);
  value operator+(const itype& v);
  value operator-(const itype& v);
  value operator*(const itype& v);
  value operator/(const itype& v);
  value operator^(const itype& v);
  value operator%(const itype& v);
  value operator-();
  value op_sin();
  value op_cos();
  value op_tan();
  value op_atan();
  value op_asin();
  value op_acos();
  value op_abs();
  value op_exp();
  value op_sqrt();
  value op_log();
  value op_cbrt();
  value cast_int();
  value cast_float();
  value op_frac();
  value op_round();
  value op_fact();
  value op_exp10();
  value op_rand();
  value op_bj(int n);
  value op_by(int n);
  int is_odd();
  int is_even();
  int is_positive();
  int is_negative();
  int is_int();
  int is_float();
  int operator<(const value& v);
  int operator>(const value& v);
  int operator<=(const value& v);
  int operator>=(const value& v);
  int operator==(const value& v);
  int operator!=(const value& v);
  static value integer(itype i);
  static value floating(ieee f);

  static value binomial(value v1, value v2);

  static ieee  dmod(ieee d1, ieee d2);
  static ieee  f_mod(ieee v1, ieee v2);
  static itype i_mod(itype i1, itype i2);

  static value intpow(itype i1, itype i2);
  static value flpow(ieee i1, itype i2);

  static void  fractionalize(ieee d, itype& p, ieee& q);
  value(uint t=INTEGER);
  void print(FILE* f,char* Fd, char* Ff);
};

//asociativnost
#define ASOC_BLANK   0x00
#define ASOC_RUNARY  0x01
#define ASOC_LUNARY  0x02
#define ASOC_RINFIX  0x03
#define ASOC_LINFIX  0x04
#define ASOC_RBINARY 0x05
#define ASOC_MULTI   0x10


#define OP_NONE 0
//memory
#define OP_TAKE 0x01
#define OP_PUT  0x02

//component-wise
#define OP_PPLUS  0x11
#define OP_MMINUS 0x12
#define OP_MMUL   0x13
#define OP_DDIV   0x14
#define OP_MMOD   0x15
#define OP_PPOW   0x16

//accumulators
#define OP_OPLUS  0x21
#define OP_OTIMES 0x22
#define OP_OMEAN  0x23
#define OP_OVAR   0x24
#define OP_ONORM  0x25

//function maps
#define OP_FACT      0x31
#define OP_MINUSSIGN 0x32
#define OP_SIN       0x33
#define OP_COS       0x34
#define OP_TAN       0x35
#define OP_ATAN      0x36
#define OP_ASIN      0x37
#define OP_ACOS      0x38
#define OP_ABS       0x39
#define OP_EXP       0x3A
#define OP_SQRT      0x3B
#define OP_CBRT      0x3C
#define OP_ROUND     0x3D
#define OP_FRAC      0x3E
#define OP_INT       0x3F
#define OP_FLOAT     0x40
#define OP_LOG       0x41
#define OP_RAND      0x42
#define OP_BESJ      0x43
#define OP_BESY      0x44

//tensor operators
#define OP_PLUS      0x51
#define OP_MINUS     0x52
#define OP_MUL       0x53
#define OP_DIV       0x54
#define OP_MOD       0x55
#define OP_POW       0x56
#define OP_FLE       0x57
#define OP_NCK       0x58

//set-wise
#define OP_AND       0x61
#define OP_WITHOUT   0x62
#define OP_OR        0x63
#define OP_XOR       0x64
#define OP_REVERSE   0x65
#define OP_SORT      0x66
#define OP_HASH      0x67
#define OP_MASK      0x68
#define OP_EQUALS    0x69
#define OP_COUNT     0x6A

//set-select
#define OP_SHL       0x71
#define OP_SHR       0x72
#define OP_INDEX     0x73
#define OP_UNIQUE    0x74
#define OP_POSITIVE  0x75
#define OP_NEGATIVE  0x76
#define OP_ODD       0x77
#define OP_EVEN      0x78
#define OP_INTEGER   0x79
#define OP_DOUBLE    0x7A
#define OP_MIN       0x7B
#define OP_MAX       0x7C

//component-wise logic
#define OP_LLT       0x81
#define OP_GGT       0x82
#define OP_LLEQ      0x83
#define OP_GGEQ      0x84
#define OP_EEQ       0x85
#define OP_NNEQ      0x86

//tensor logic
#define OP_LT        0x91
#define OP_GT        0x92
#define OP_LEQ       0x93
#define OP_GEQ       0x94
#define OP_EQ        0x95
#define OP_NEQ       0x96

//const
#define OP_PI        0xA1
#define OP_AA        0xA2
#define OP_DEG       0xA3
#define OP_PM        0xA4
#define OP_MP        0xA5
#define OP_EMC       0xA6

//polynomial
#define OP_XPLUS  0xB1
#define OP_XMINUS 0xB2
#define OP_XMUL   0xB3
#define OP_XDIV   0xB4
#define OP_XMOD   0xB5
#define OP_XPOW   0xB6
#define OP_PX     0xB7
#define OP_PD     0xB8
#define OP_PZ     0xB9

//vectors
#define OP_DOT    0xC1
#define OP_VLEN   0xC2
#define OP_VNRM   0xC3
#define OP_CROSS  0xC4

#define OP_LIST      0xFF
#define OP_ITER1     0xFE
#define OP_ITER2     0xFD

struct opid {
  char* name;
  uchar code;
  uchar precedence;
  uchar asoc;
  opid(char* n, uchar c, uchar p, uchar a);
  opid();
};

#define STATE_IDLE 0
#define STATE_WORK 1

namespace shared {
  extern vector<opid> shared_ops;
#ifdef SIGHANDLE
  extern sig_atomic_t interactive_flag;
  extern sig_atomic_t state;
  extern sig_atomic_t interrupt;
  extern char* wait_ptr;
#endif
}

opid findop(char* c);

void fill_ops();
void kill_ops();


#endif
