/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "tokener.h"

#include <cctype>
#include <cstring>

using namespace std;

static char* makestring(vector<char>& buffer){
  char* ch=new char[buffer.size()+1];
  for(uint i=0;i<buffer.size();i++){
    ch[i]=buffer[i];
  }
  ch[buffer.size()]=0;
  buffer.clear();
  return ch;
}


token::token(){
  type=TOK_NONE;
}

token::token(uchar t){
  type=t;
}

token token::val(value f){
  token tok(TOK_VALUE);
  tok.f=f;
  return tok;
}

token token::op(opid op){
  token tok(TOK_OP);
  tok.o=op;
  return tok;
}

token token::parl(uchar type){
  token tok(TOK_PAR_L);
  tok.i=type;
  return tok;
}

token token::parr(uchar type){
  token tok(TOK_PAR_R);
  tok.i=type;
  return tok;
}

#if DEBUG>2
void printtoken(token& tok){
  switch(tok.type){
    case TOK_NONE: printf("none "); break;
    case TOK_PAR_L:
      switch(tok.i){
        case OP_LIST: printf("{ ");break;
        case OP_ITER1: printf("[ ");break;
        case OP_ITER2: printf("( ");break;
      }
      break;
    case TOK_PAR_R:
      switch(tok.i){
        case OP_LIST: printf("} ");break;
        case OP_ITER1: printf("] ");break;
        case OP_ITER2: printf(") ");break;
      }
      break;
    case TOK_OP:
      if(tok.o.code==OP_MINUSSIGN)
        printf("-");
      else
        printf("%s ",tok.o.name);
      break;
    case TOK_VALUE:
      tok.f.print(stdout,"%d ","%g ");
      break;
  }
}
#endif


void tokenize(char* source, vector<token>& tokens){
  uchar state=STATE_TOK_BETWEEN;//we start in between words
  vector<char> buffer;
  int c;

  int whitespace=1;
  uint seek=0;//string position (input is already separated into lines)

  while(state){
    c=source[seek];
    #if DEBUG>2
    fprintf(stderr,"STATE_TOK %d %c\n",state,c);
    #endif
    switch(state){
      case STATE_TOK_BETWEEN: {//not inside word or number
        if(c==0){state=STATE_TOK_END; break;}
        if(c=='-'){
          if(!tokens.size()){tokens.push_back(token::op(shared::shared_ops[3]));break;}
          if(whitespace&&((!isspace(source[seek+1]))&&(source[seek+1]!=',')&&(source[seek+1]!='-')))
          {tokens.push_back(token::op(shared::shared_ops[3]));break;}
          token last=tokens[tokens.size()-1];
          //after closing brackets and operators that don't need right argument, this is unary minus
          if(((last.type==TOK_OP) && (last.o.asoc!=ASOC_LUNARY) && (last.o.asoc!=ASOC_BLANK))||last.type==TOK_PAR_L){tokens.push_back(token::op(shared::shared_ops[3]));break;}
        }
        if(c=='+'){
          if(!tokens.size())break;
          if(whitespace&&((!isspace(source[seek+1]))&&(source[seek+1]!=',')&&(source[seek+1]!='+')))break;
          token last=tokens[tokens.size()-1];
          if(((last.type==TOK_OP) && (last.o.asoc!=ASOC_LUNARY) && (last.o.asoc!=ASOC_BLANK))||last.type==TOK_PAR_L)break;
        }
        opid o=findop(source+seek);
        if(o.code!=OP_NONE){
          tokens.push_back(token::op(o));
          seek+=strlen(o.name)-1;
          break;
        }
        if(isspace(c)||(c==','))break;
        if(c=='{'){
          tokens.push_back(token::parl(OP_LIST));
          break;
        }
        if(c=='}'){
          tokens.push_back(token::parr(OP_LIST));
          break;
        }
        if(c=='['){
          tokens.push_back(token::parl(OP_ITER1));
          break;
        }
        if(c==']'){
          tokens.push_back(token::parr(OP_ITER1));
          break;
        }
        if(c=='('){
          tokens.push_back(token::parl(OP_ITER2));
          break;
        }
        if(c==')'){
          tokens.push_back(token::parr(OP_ITER2));
          break;
        }
        if(isdigit(c)||(c=='.')){
          buffer.push_back(c);
          state=STATE_TOK_NUM;
          break;
        }
        //nothing matched
        fprintf(stderr,"parser: skipping bogus character '%c'\n",c);
        break;
      }
      case STATE_TOK_NUM: {//reading a number
        if(isdigit(c)||(c=='.')){buffer.push_back(c);break;}
        if(!(isspace(c)||(c==',')))seek--;
        char* val=makestring(buffer);
        if(strchr(val,'.')){
          #if VALUE==LONG
          value x=value::floating(strtold(val,0));
          #elif VALUE==DOUBLE
          value x=value::floating(strtod(val,0));
          #else
          value x=value::floating(strtof(val,0));
          #endif
          tokens.push_back(token::val(x));
        }else{
          #if IVALUE==ILONG
          value x=value::integer(strtoll(val,0,10));
          #else
          value x=value::integer(strtol(val,0,10));
          #endif
          tokens.push_back(token::val(x));
        }
        delete[] val;
        state=STATE_TOK_BETWEEN;
        break;
      }
    }
    whitespace=isspace(c)||(c==',');
    ++seek;
  }
  buffer.clear();
}

