/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "vm.h"
#include "methods.h"

#ifdef TIMERAND
#include <ctime>
#endif

#include <algorithm>
#include <cstring>
#include <cctype>

using namespace std;

array eval_control::evaluate_value(value v){
  array a;
  a.push_back(v);
  return a;
}

struct comparator {
  int operator()(const value v1, const value v2){
    return v1.d<v2.d;
  }
};

//functions

#define a  args[0]
#define a1 args[0]
#define a2 args[1]

array eval_control::evaluate_operator(uchar code, vector<array>& args){
  array out;
  switch(code){
    case OP_NONE: return out;
    case OP_TAKE: {
      int n=a.size();
      for(int i=0;i<n;i++){
        array mem;
        int index=a[i].assertI("${n}");
        if(index<0){
          if(-index<=history.size()){
            mem=history[history.size()+index];
          }else continue;
        }else{
          mem=memory[index];
        }
        int n2=mem.size();
        for(int j=0;j<n2;j++){
          out.push_back(mem[j]);
        }
      }
      return out;
    }
    case OP_PUT:{
      int n1=a1.size();
      for(int i=0;i<n1;i++){
         memory[a1[i].assertI("@{n}{...}")]=a2;
      }
      out=a2;
      return out;
    }

    case OP_PPLUS:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a1[i]+a2[i]);
      }
      return out;
    }
    case OP_MMINUS:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a1[i]-a2[i]);
      }
      return out;
    }
    case OP_MMUL:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a1[i]*a2[i]);
      }
      return out;
    }
    case OP_DDIV:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a1[i]/a2[i]);
      }
      return out;
    }
    case OP_MMOD:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a1[i]%a2[i]);
      }
      return out;
    }
    case OP_PPOW:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a1[i]^a2[i]);
      }
      return out;
    }


    case OP_OPLUS:{
      int n=a.size();
      value v=value::integer(0);
      for(int i=0;i<n;i++){
        v=v+a[i];
      }
      out.push_back(v);
      return out;
    }
    case OP_OTIMES:{
      int n=a.size();
      value v=value::integer(1);
      for(int i=0;i<n;i++){
        v=v*a[i];
      }
      out.push_back(v);
      return out;
    }
    case OP_OMEAN:{
      int n=a.size();
      value v=value::floating(0);
      for(int i=0;i<n;i++){
        v=v+a[i];
      }
      out.push_back(v/(itype)n);
      return out;
    }
    case OP_OVAR:{
      int n=a.size();
      value v=value::floating(0);
      value v2=value::floating(0);
      for(int i=0;i<n;i++){
        v=v+a[i];
        v2=v2+(a[i]*a[i]);
      }
      out.push_back(v2/(itype)n-((v/(itype)n)^(itype)2));
      return out;
    }
    case OP_ONORM:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n1;i++){
        value v=value::integer(0);
        for(int j=0;j<n2;j++){
          v=v+((a2[j].op_abs())^a1[i]);
        }
        out.push_back(v^(value::floating(1.)/a1[i]));
      }
      return out;
    }


    case OP_FACT:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_fact());
      }
      return out;
    }
    case OP_MINUSSIGN:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(-a[i]);
      }
      return out;
    }
    case OP_SIN:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_sin());
      }
      return out;
    }
    case OP_COS:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_cos());
      }
      return out;
    }
    case OP_TAN:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_tan());
      }
      return out;
    }
    case OP_ATAN:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_atan());
      }
      return out;
    }
    case OP_ASIN:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_asin());
      }
      return out;
    }
    case OP_ACOS:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_acos());
      }
      return out;
    }
    case OP_ABS:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_abs());
      }
      return out;
    }
    case OP_EXP:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_exp());
      }
      return out;
    }
    case OP_SQRT:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_sqrt());
      }
      return out;
    }
    case OP_CBRT:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_cbrt());
      }
      return out;
    }
    case OP_ROUND:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_round());
      }
      return out;
    }
    case OP_FRAC:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_frac());
      }
      return out;
    }
    case OP_INT:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].cast_int());
      }
      return out;
    }
    case OP_FLOAT:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].cast_float());
      }
      return out;
    }
    case OP_LOG:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_log());
      }
      return out;
    }
    case OP_RAND:{
      int n=a.size();
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(a[i].op_rand());
      }
      return out;
    }
    case OP_BESJ:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        int besn=a1[i].assertI("bj{n}{...}");
        for(int j=0;j<n2;j++){
          out.push_back(a2[j].op_bj(besn));
        }
      }
      return out;
    }
    case OP_BESY:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        int besn=a1[i].assertI("by{n}{...}");
        for(int j=0;j<n2;j++){
          out.push_back(a2[j].op_by(besn));
        }
      }
      return out;
    }


    case OP_PLUS:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]+a2[j]);
        }
      }
      return out;
    }
    case OP_MINUS:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]-a2[j]);
        }
      }
      return out;
    }
    case OP_MUL:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]*a2[j]);
        }
      }
      return out;
    }
    case OP_DIV:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]/a2[j]);
        }
      }
      return out;
    }
    case OP_MOD:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]%a2[j]);
        }
      }
      return out;
    }
    case OP_POW:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]^a2[j]);
        }
      }
      return out;
    }
    case OP_FLE:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(a1[i]*a2[j].op_exp10());
        }
      }
      return out;
    }
    case OP_NCK:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::binomial(a1[i],a2[j]));
        }
      }
      return out;
    }

    case OP_AND:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n1;i++){
        int b=0;
        for(int j=0;j<n2;j++){
          if(a1[i]==a2[j]){b=1;break;}
        }
        if(b)out.push_back(a1[i]);
      }
      return out;
    }
    case OP_WITHOUT:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n1;i++){
        int b=1;
        for(int j=0;j<n2;j++){
          if(a1[i]==a2[j]){b=0;break;}
        }
        if(b)out.push_back(a1[i]);
      }
      return out;
    }
    case OP_OR:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n1;i++){
        out.push_back(a1[i]);
      }
      for(int j=0;j<n2;j++){
        out.push_back(a2[j]);
      }
      return out;
    }
    case OP_XOR:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n1;i++){
        int b=1;
        for(int j=0;j<n2;j++){
          if(a1[i]==a2[j]){b=0;break;}
        }
        if(b)out.push_back(a1[i]);
      }
      for(int i=0;i<n2;i++){
        int b=1;
        for(int j=0;j<n1;j++){
          if(a1[j]==a2[i]){b=0;break;}
        }
        if(b)out.push_back(a2[i]);
      }
      return out;
    }
    case OP_REVERSE:{
      int n=a.size();
      out.reserve(n);
      for(int i=n-1;i>=0;i--){
        out.push_back(a[i]);
      }
      return out;
    }
    case OP_SORT:{
      out=a;
      sort(out.begin(),out.end(),comparator());
      return out;
    }
    case OP_HASH:{
      int n=a.size();
      out.push_back(value::integer(n));
      return out;
    }
    case OP_MASK:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i]!=value::integer(0)){
          out.push_back(value::integer((itype)i));
        }
      }
      return out;
    }
    case OP_EQUALS:{
      int n1=a1.size();
      int n2=a2.size();
      if(n1!=n2){
        out.push_back(value::integer(0));
        return out;
      }else{
        for(int i=0;i<n1;i++){
          if(a1[i]!=a2[i]){
            out.push_back(value::integer(0));
            return out;
          }
        }
        out.push_back(value::integer(1));
        return out;
      }
    }
    case OP_COUNT:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n1;i++){
        int count=0;
        for(int j=0;j<n2;j++){
          if(a1[i]==a2[j])count++;
        }
        out.push_back(value::integer((itype)count));
      }
      return out;
    }

    case OP_SHL:{
      int n1=a1.size();
      int n2=a2.size();
      for(int j=0;j<n2;j++){
        int ihi=a2[j].assertI("{...}<<{n}");
        int i1=ihi<0?-ihi:0;
        int i2=ihi<0?n1:ihi;
        for(int i=i1;i<i2;i++){
          if((i<0)||(i>=n1)){out.push_back(value::integer(0));continue;}
          out.push_back(a1[i]);
        }
      }
      return out;
    }
    case OP_SHR:{
      int n1=a1.size();
      int n2=a2.size();
      for(int j=0;j<n2;j++){
        int ihi=a2[j].assertI("{...}>>{n}");
        int i1=ihi<0?0:n1-ihi;
        int i2=ihi<0?n1+ihi:n1;
        for(int i=i1;i<i2;i++){
          if((i<0)||(i>=n1)){out.push_back(value::integer(0));continue;}
          out.push_back(a1[i]);
        }
      }
      return out;
    }
    case OP_INDEX:{
      int n1=a1.size();
      if(!n1)return out;
      int n2=a2.size();

      out.reserve(n2);
      for(int j=0;j<n2;j++){
        value v=a2[j]%(itype)n1;
        if(v.type==INTEGER){
          out.push_back(a1[v.i]);
        }else{
          out.push_back(a1[v.i]*((ieee)1.-v.f)+a1[(v.i+1)%n1]*v.f);
        }
      }
      return out;
    }
    case OP_UNIQUE:{
      int n=a.size();
      for(int i=0;i<n;i++){
        int b=1;
        for(int j=0;j<out.size();j++){
          if(out[j]==a[i]){b=0;break;}
        }
        if(b)out.push_back(a[i]);
      }
      return out;
    }
    case OP_POSITIVE:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i].is_positive())out.push_back(a[i]);
      }
      return out;
    }
    case OP_NEGATIVE:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i].is_negative())out.push_back(a[i]);
      }
      return out;
    }
    case OP_ODD:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i].is_odd())out.push_back(a[i]);
      }
      return out;
    }
    case OP_EVEN:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i].is_even())out.push_back(a[i]);
      }
      return out;
    }
    case OP_INTEGER:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i].is_int())out.push_back(a[i]);
      }
      return out;
    }
    case OP_DOUBLE:{
      int n=a.size();
      for(int i=0;i<n;i++){
        if(a[i].is_float())out.push_back(a[i]);
      }
      return out;
    }
    case OP_MIN:{
      int n=a.size();
      if(!n)return out;
      value v=a[0];
      for(int i=1;i<n;i++){
        if(a[i]<v)v=a[i];
      }
      out.push_back(v);
      return out;
    }
    case OP_MAX:{
      int n=a.size();
      if(!n)return out;
      value v=a[0];
      for(int i=1;i<n;i++){
        if(a[i]>v)v=a[i];
      }
      out.push_back(v);
      return out;
    }

    case OP_LLT:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(value::integer(a1[i]<a2[i]));
      }
      return out;
    }
    case OP_GGT:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(value::integer(a1[i]>a2[i]));
      }
      return out;
    }
    case OP_LLEQ:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(value::integer(a1[i]<=a2[i]));
      }
      return out;
    }
    case OP_GGEQ:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(value::integer(a1[i]>=a2[i]));
      }
      return out;
    }
    case OP_EEQ:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(value::integer(a1[i]==a2[i]));
      }
      return out;
    }
    case OP_NNEQ:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      for(int i=0;i<n;i++){
        out.push_back(value::integer(a1[i]!=a2[i]));
      }
      return out;
    }

    case OP_LT:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::integer(a1[i]<a2[j]));
        }
      }
      return out;
    }
    case OP_GT:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::integer(a1[i]>a2[j]));
        }
      }
      return out;
    }
    case OP_LEQ:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::integer(a1[i]<=a2[j]));
        }
      }
      return out;
    }
    case OP_GEQ:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::integer(a1[i]>=a2[j]));
        }
      }
      return out;
    }
    case OP_EQ:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::integer(a1[i]==a2[j]));
        }
      }
      return out;
    }
    case OP_NEQ:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      for(int i=0;i<n1;i++){
        for(int j=0;j<n2;j++){
          out.push_back(value::integer(a1[i]!=a2[j]));
        }
      }
      return out;
    }

    case OP_PI:{
      out.push_back(value::floating(PI));
      return out;
    }
    case OP_AA:{
      out.push_back(value::floating(AA));
      return out;
    }
    case OP_DEG:{
      out.push_back(value::floating(DEG));
      return out;
    }
    case OP_EMC:{
      out.push_back(value::floating(EMC));
      return out;
    }
    case OP_PM:{
      out.push_back(value::integer(1));
      out.push_back(value::integer(-1));
      return out;
    }
    case OP_MP:{
      out.push_back(value::integer(-1));
      out.push_back(value::integer(1));
      return out;
    }

    case OP_XPLUS:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      int nx=n1>n2?n1:n2;
      for(int i=0;i<n;i++){
        out.push_back(a1[i]+a2[i]);
      }
      for(int i=n;i<nx;i++){
        if(n1>n2){
          out.push_back(a1[i]);
        }else{
          out.push_back(a2[i]);
        }
      }
      return out;
    }
    case OP_XMINUS:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      int nx=n1>n2?n1:n2;
      for(int i=0;i<n;i++){
        out.push_back(a1[i]-a2[i]);
      }
      for(int i=n;i<nx;i++){
        if(n1>n2){
          out.push_back(a1[i]);
        }else{
          out.push_back(-a2[i]);
        }
      }
      return out;
    }
    case OP_XMUL:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1+n2-1;
      for(int i=0;i<n;i++){
        value v=value::integer(0);
        for(int j=0;j<=i;j++){
          if((j<n1)&&(i-j<n2)){
            v=v+a1[j]*a2[i-j];
          }
        }
        out.push_back(v);
      }
      return out;
    }
    case OP_XDIV:{
      int n1=a1.size();
      int n2=a2.size();
      //trim the zeroes
      for(int i=n1-1;i>=0;i--){
        if(a1[i].d==0){
          n1=i;
        }else break;
      }
      for(int i=n2-1;i>=0;i--){
        if(a2[i].d==0){
          n2=i;
        }else break;
      }
      if((n1==0) || (n2==0)){
        return out;
      }
      if(n1<n2){
        out.push_back(value::integer(0));
        return out;
      }
      if(n1==n2){//trivial solution, when polynomials have the same order
        out.push_back(value::floating(a1[n1-1].d/a2[n2-1].d));
        return out;
      }
      int n=n1-n2+1;
      out=array(a1.begin(),a1.begin()+n1);
      for(int i=n-1;i>=0;i--){
        if(out[i+n2-1].d==0)continue;
        out[i+n2-1]=out[i+n2-1]/a2[n2-1].d;
        for(int j=0;j<n2-1;j++){
          out[i+j]=out[i+j]-a2[j]*out[i+n2-1].d;
        }
      }
      out.erase(out.begin(),out.begin()+(n1-n));
      return out;
    }
    case OP_XMOD:{
      int n1=a1.size();
      int n2=a2.size();
      //trim the zeroes
      for(int i=n1-1;i>=0;i--){
        if(a1[i].d==0){
          n1=i;
        }else break;
      }
      for(int i=n2-1;i>=0;i--){
        if(a2[i].d==0){
          n2=i;
        }else break;
      }
      if((n1==0) || (n2==0)){
        return out;
      }
      if(n2==1){//always divisible
        out.push_back(value::integer(0));
        return out;
      }
      if(n1<n2){//not divisible
        out=a1;
        return out;
      }
      int n=n1-n2+1;
      out=array(a1.begin(),a1.begin()+n1);
      for(int i=n-1;i>=0;i--){
        if(out[i+n2-1].d==0)continue;
        out[i+n2-1]=out[i+n2-1]/a2[n2-1].d;
        for(int j=0;j<n2-1;j++){
          out[i+j]=out[i+j]-a2[j]*out[i+n2-1].d;
        }
      }
      out.erase(out.begin()+(n1-n),out.end());
      return out;
    }

    case OP_PX:{
      int n1=a1.size();
      int n2=a2.size();
      for(int i=0;i<n2;i++){
        value v=a1[n1-1];
        for(int j=n1-2;j>=0;j--){
          v=v*a2[i]+a1[j];
        }
        out.push_back(v);
      }
      return out;
    }
    case OP_PD:{
       int n=a.size();
       for(int i=1;i<n;i++){
         out.push_back(a[i]*(itype)i);
       }
       return out;
    }
    case OP_PZ:{
       bairstow(a,out);
       return out;
    }

    case OP_DOT:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      value v=value::integer(0);
      for(int i=0;i<n;i++){
        v=v+a1[i]*a2[i];
      }
      out.push_back(v);
      return out;
    }
    case OP_VLEN:{
      int n=a.size();
      value v=value::integer(0);
      for(int i=0;i<n;i++){
        v=v+(a[i]*a[i]);
      }
      out.push_back(v.op_sqrt());
      return out;
    }
    case OP_VNRM:{
      int n=a.size();
      value v=value::integer(0);
      for(int i=0;i<n;i++){
        v=v+(a[i]*a[i]);
      }
      v=v.op_sqrt();
      for(int i=0;i<n;i++){
        out.push_back(a[i]/v);
      }
      return out;
    }
    case OP_CROSS:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      if(n==1){out.push_back(value::integer(0));return out;}
      for(int i=0;i<n;i++){
        out.push_back(a1[(i+1)%n1]*a2[(i+2)%n2]-a1[(i+2)%n1]*a2[(i+1)%n2]);
      }
      return out;
    }

    case OP_LIST:{
      for(int i=0;i<args.size();i++){
        for(int j=0;j<args[i].size();j++){
          out.push_back(args[i][j]);
        }
      }
      return out;
    }
    case OP_ITER1:{
      array arg;
      for(int i=0;i<args.size();i++){//collect
        for(int j=0;j<args[i].size();j++){
          arg.push_back(args[i][j]);
        }
      }
      int n=arg.size();
      switch(n){
        case 0:{
          out.push_back(value::integer(42));
          break;
          }
        case 1:{
          int count=arg[0].assertI("[n]");
          for(int i=0;i<count;i++){
            out.push_back(value::integer(i));
          }
          break;
          }
        case 2:{
          if(arg[1]>arg[0]){
            for(value v=arg[0];v<=arg[1];v=v+(itype)1){
              out.push_back(v);
            }
          }else{
            for(value v=arg[0];v>=arg[1];v=v-(itype)1){
              out.push_back(v);
            }
          }
          break;
          }
        default:{
          //same value?
          if(arg[0].d==arg[n-1].d){
            out.push_back(arg[0]);
            break;
          }
          value max=value::integer(0); //max. offset offset
          value min=value::integer(0);
          value sum=value::integer(0);
          for(int i=1;i<n-1;i++){
            sum=sum+arg[i];
            max=sum>max?sum:max;
            min=sum<min?sum:min;
          }

          //is there minus?
          if(arg[0]>arg[n-1]){//if the sequence is descending or if it overshot final value
            if((sum.d<0)||(min<arg[n-1]-arg[0])){
              value v=arg[0];
              for(int i=0;v>=arg[n-1];i++){
                out.push_back(v);
                v=v+arg[1+i%(n-2)];
              }
            }
          }else{//if the sequence is ascending or if it overshot final value
            if((sum.d>0)||(max>arg[n-1]-arg[0])){
              value v=arg[0];
              for(int i=0;v<=arg[n-1];i++){
                out.push_back(v);
                v=v+arg[1+i%(n-2)];
              }
            }
          }
          break;
          }
      }
      return out;
    }
    case OP_ITER2:{
      array arg;
      for(int i=0;i<args.size();i++){//collect
        for(int j=0;j<args[i].size();j++){
          arg.push_back(args[i][j]);
        }
      }
      int n=arg.size();
      switch(n){
        case 0:break;
        case 1:{
          out.push_back(arg[0]);
          break;
          }
        case 2:{
          for(int i=0;i<arg[1].d;i++){
            out.push_back(arg[0]);
          }
          break;
          }
        default:{
          value v=arg[0];
          ieee count=arg[n-1].d;
          if(count<0)count=(-count)*(n-2)+1;//negative: whole sequence times count
          out.reserve((int)(count+0.5));
          for(int i=0;i<count;i++){
            out.push_back(v);
            v=v+arg[1+i%(n-2)];
          }
          break;
          }
      }
      return out;
    }


  }
}

#undef a
#undef a1
#undef a2

array eval_control::evaluate(treenode& node){
  switch(node.type){
    case NODE_OP:{
      vector<array> args;
      for(list<treenode>::iterator i=node.childz.begin();i!=node.childz.end();i++){
#ifdef SIGHANDLE
        //interrupt
        if(shared::interrupt){return array();}
#endif
        args.push_back(evaluate(*i));//recursion!!!
      }
      return evaluate_operator(node.op.code,args);
    }
    case NODE_VAL:{
      return evaluate_value(node.val);
    }
  }
}

void eval_control::print_array(array& a, FILE* stream, const char* sep, char end){
    if(!a.size()){if(end)fputc(end,stream);return;}
    for(int i=0;i<a.size()-1;i++){
      a[i].print(stream,Fd,Ff);
      if(sep)fputs(sep,stream);
#ifdef SIGHANDLE
    //interrupt - long outputs take forever, must be able to break loop
    if(shared::interrupt){fprintf(stream,"\n");return;}
#endif
    }
    a[a.size()-1].print(stream,Fd,Ff);
    if(end)fputc(end,stream);
}

static char* strip(char* str){
  char* out=(char*)malloc(strlen(str)+1);
  char* pos=out;//current character
  char* end=out;//position of null
  char mode=0; //quote type
  for(;*str;str++){
    if(!mode){
      if(isspace(*str))continue;
      if(*str=='\''){mode='\'';continue;}
      if(*str=='"'){mode='"';continue;}
      if(*str=='\\'){mode=' ';str--;continue;}//we handle escapes later
      mode=' ';
      *pos=*str;pos++;end=pos;
      continue;
    }
    //escaped characters (not for '')
    if((*str=='\\')&&(mode!='\'')){
      switch(str[1]){
        case 0: break;
        case 't': *pos='\t';pos++;break;
        case 'n': *pos='\n';pos++;break;
        case 'r': *pos='\r';pos++;break;
        case 'f': *pos='\f';pos++;break;
        case 'b': *pos='\b';pos++;break;
        case 'v': *pos='\v';pos++;break;
        default: *pos=str[1];pos++;
      }
      //escaped characters count even at the end
      end=pos;
      if(str[1]){
        str++;
      }else{//null after escape
        fprintf(stderr,"input: escape character at the end of input\n");
      }
      continue;
    }
    if((mode!=' ')&&(*str==mode)){
      mode=' ';//reset mode
      break;
    }
    *pos=*str;pos++;
    //if no quotes, spaces at the end don't count
    if(!((mode==' ')&&isspace(*str)))end=pos;
  }
  if(!mode){//etirely empty line
    free(out);
    return 0;
  }
  if(mode!=' '){//there was no closing quote
   fprintf(stderr,"input: missing closing quote '%c'\n",mode);
  }
  *end=0;//put the null where the string ends
  return out;
}

FILE* eval_control::file_open(char* filename){
  if(!strcmp(filename,"-"))return stdin;
  FILE* f=fopen(filename,"r");
  if(f)return f;
  fprintf(stderr,"input: file '%s' could not be opened\n",filename);
  return 0;
}

array eval_control::evaluate_code(vector<treenode>& code,int semicoloned){
  for(int i=0;i<code.size()-(1-semicoloned);i++){//everything but unsemicoloned part
    array a=evaluate(code[i]);
    //print if forced print is enabled
    if(toggle_print)print_array(a,stdout,delim,'\n');
  }
  //last part, if not semicoloned
  if((!semicoloned)&&(code.size()>0)){
    return evaluate(code.back());
  }
  return array();
}

void eval_control::precompile(char* line, char** comment,vector<treenode>& code, int& semicoloned){
  semicoloned=1;//so far, it terminates with semicolon
  //shabang lines should be ignored. This way you can call ised with
  //#!/usr/local/bin/ised --f
  char* shabang=strstr(line,"#!");
  //if line begins with #!, omit it.
  //if line is empty, omit it.
  if((shabang==line)||(*line=='\n')||(*line==0)){if(comment)*comment=0;return;}
  //if there's no comment, comment is end of string.
  if(comment)*comment=shabang?shabang:strchr(line,0);
  if(shabang)*shabang=0;//premature end of line at comment

  char* end;
  while(end=strchr(line,';')){//semicolon separates chunks of code
#ifdef SIGHANDLE
    //interrupt
    if(shared::interrupt){if(comment)*comment=0;return;}
#endif
    *end=0;//fake end
    vector<token> tokens;
    tokenize(line,tokens);
    compiler c;
    code.push_back(c.compile(tokens));
    *end=';';//restore end
    line=end+1;
  }
  //if nothing remains
  if((*line=='\n')||(*line==0)){return;}
  //finalization
  vector<token> tokens;
  tokenize(line,tokens);
  if(shabang)*shabang='#';//restore original line
  compiler c;
  code.push_back(c.compile(tokens));
  semicoloned=0;//if there's more code after last semicolon
}

char* eval_control::bufgets(FILE* file){
   char* out=0;
   //put some dummy non-null character
   buffer[bufsize-1]=' ';
   out=fgets(buffer,bufsize,file);
   //if it changed, the buffer was too short
   while(buffer[bufsize-1]!=' '){
     #if DEBUG>0
     fprintf(stderr,"input: buffer resize from %d to %d\n",bufsize,2*bufsize);
     #endif
     char* newbuf=(char*)malloc(2*bufsize);
     memcpy(newbuf,buffer,bufsize);
     free(buffer);
     buffer=newbuf;
     buffer[2*bufsize-1]=' ';
     out=fgets(buffer+bufsize-1,bufsize+1,file);
     bufsize*=2;
   }
   return out;
}

#ifndef HAVE_LIBREADLINE
char* readline(const char* prompt){
  fputs(prompt,stdout);
  char* buf=(char*)malloc(BUF_SIZE);
  return fgets(buf,BUF_SIZE,stdin);
}
#endif

void eval_control::read_interactive(){
  credits_dump();
#ifdef SIGHANDLE
  shared::wait_ptr=&awaiting;
#endif
  char prompt[7];
  strcpy(prompt,"ised> ");
  char* line;
  while(line=readline(prompt)){
#ifdef SIGHANDLE
    shared::state=STATE_WORK;
#endif
    input_line(line);
#ifdef SIGHANDLE
    shared::state=STATE_IDLE;
    shared::interrupt=0;
#endif
#ifdef HAVE_READLINE_HISTORY
    if(strlen(line))add_history(line);
#endif
    free(line);
    //change prompt:
    if(awaiting){
      sprintf(prompt,"%c= ",awaiting);
    }else{
      strcpy(prompt,"ised> ");
    }
  }
#ifdef SIGHANDLE
  shared::wait_ptr=0;
#endif
}

array eval_control::evaluate_line(char* line, char** comment){
  vector<treenode> code;
  int semicoloned;
  precompile(line,comment,code,semicoloned);
  if(semicoloned){if(comment)*comment=0;}
  return evaluate_code(code,semicoloned);
}


void eval_control::read_file(FILE* f){
  while(bufgets(f)){
    input_line(buffer);
  }
}

void eval_control::eval_file(FILE* f){
  char* comment;
  array a;
  while(bufgets(f)){
    a=evaluate_line(buffer,&comment);
    //print unless it's commented or empty line
    if(comment)print_array(a,stdout,delim,'\n');
  }
  history.push_back(a);//last line gets into history
}

void eval_control::load_file(FILE* f){
  char* comment;
  array a;
  while(bufgets(f)){
    a=evaluate_line(buffer,&comment);
    //save to memory unless it's commented or empty line
    if(comment)memory[++mempos]=a;
  }
  memory[0]=evaluate_value(value::integer(mempos));
}

void eval_control::apply_line_to_file(char* f, FILE* l){
  char* comment;

  vector<treenode> code;
  int semicoloned;
  precompile(f,&comment,code,semicoloned);
  if(!comment)return;//empty, failed or entirely commented
  int line=1;
  while(bufgets(l)){
    memory[0]=evaluate_value(value::integer(line));
    memory[1]=evaluate_line(buffer,&comment);
    array a=evaluate_code(code,semicoloned);
    if(!semicoloned){//only print if there's something to print
        if(comment){
        line++;
        history.push_back(a);
        print_array(a,stdout,delim,0);
        if(*comment){
            fputs(comment,stdout);
        }else{
            fputc('\n',stdout);
        }
        }else{//if empty, failed or entirely commented, preserve it (preserves blanks)
        fputs(buffer,stdout);
        }
    }
  }
}

void eval_control::interlace_code(vector<treenode>& code){
  vector<array> temp_buffer;
  for(int i=0;i<code.size();i++){
    temp_buffer.push_back(evaluate(code[i]));
  }

  int index=0;
  while(1){
    array a;
    for(int j=0;j<temp_buffer.size();j++){
      if(temp_buffer[j].size()>index){
        a.push_back(temp_buffer[j][index]);
      }
    }
    if(a.size()>0){
      print_array(a,stdout,delim,'\n');
      index++;
    }else break;
  }
}

void eval_control::interlace_line(char* line){
  vector<treenode> code;
  int semicoloned;
  precompile(line,0,code,semicoloned);
  interlace_code(code);
}

void eval_control::apply_file_to_file(FILE* f, FILE* l){
  char* comment;

  int line=1;
  while(bufgets(l)){
    memory[0]=evaluate_value(value::integer(line));
    memory[1]=evaluate_line(buffer,&comment);
    if(comment){
      line++;
      eval_file(f);
      rewind(f);
    }else{
      fputs(buffer,stdout);
    }
  }
}

void eval_control::interlace_to_file(char* f, FILE* l){
  vector<treenode> code;
  int semicoloned;
  precompile(f,0,code,semicoloned);

  char* comment;

  int line=1;
  while(bufgets(l)){
    memory[0]=evaluate_value(value::integer(line));
    memory[1]=evaluate_line(buffer,&comment);
    if(comment){
      line++;
      interlace_code(code);
    }else{
      fputs(buffer,stdout);
    }
  }
}

void eval_control::memory_dump(){
  typedef map<int,array>::iterator iterator;
  fprintf(stderr,"memory page:\n");
  for(iterator i=memory.begin();i!=memory.end();i++){
    fprintf(stderr,"@%+6d | ",(*i).first);
    array a=(*i).second;
    print_array(a,stderr," ",'\n');
  }
  fprintf(stderr,"end of memory page\n");
}

void eval_control::credits_dump(){
fprintf(stderr,"%s  Copyright (C) 2009-2010 Simon Copar\n\
This program comes with ABSOLUTELY NO WARRANTY.\n\
This is free software, and you are welcome to redistribute it\n\
under certain conditions.\n",
PACKAGE_STRING);
}

void eval_control::configure_dump(){
#if VALUE==FLOAT
const char* dstr="FLOAT";
#elif VALUE==DOUBLE
const char* dstr="DOUBLE";
#elif VALUE==LONG
const char* dstr="LONG DOUBLE";
#endif

#if IVALUE==IINT
const char* istr="INT";
#elif IVALUE==ILONG
const char* istr="LONG LONG";
#endif
fprintf(stderr,"Build parameters:\n\
  floating point precision=%s (%d bytes)\n\
  integer precision=%s (%d bytes)\n",
  dstr,sizeof(ieee),istr,sizeof(itype));
#ifndef HAVE_LIBREADLINE
const char* lrlstr="DISABLED";
#else
const char* lrlstr="ENABLED";
#endif
#ifndef HAVE_READLINE_HISTORY
const char* rlhstr="DISABLED";
#else
const char* rlhstr="ENABLED";
#endif
fprintf(stderr,"  libreadline %s, history %s\n", lrlstr,rlhstr);
}

void eval_control::input_line(char* line){
  char* tofree=0;
  if(!awaiting){
    if(!memcmp(line,"--m",3)){memory_dump();return;}
    if(!memcmp(line,"--p",3)){toggle_print=1;return;}
    if(!memcmp(line,"--n",3)){toggle_print=0;return;}
    if(!memcmp(line,"--v",3)){credits_dump();putc('\n',stderr);configure_dump();return;}
    const char* chars="faldtDF";
    char compare[4]="---";

    while(*chars){
      compare[2]=*chars;
      if(!memcmp(line,compare,3)){
        tofree=line=strip(line+3);
        awaiting=*chars;
        break;
      }
      chars++;
    }
  }

  //waiting for something
  if(awaiting){
    if((!tofree)&&line)tofree=line=strip(line);
    if(!line)return;//really empty line
    switch(awaiting){
      case 'f':{
        awaiting=0;
        FILE* f=file_open(line);
        if(f){
          if(lfile){//if we want to apply to another file
            FILE* l=file_open(lfile);
            if(l){
              apply_file_to_file(f,l);
              fclose(l);
            }
            free(lfile);
            lfile=0;
          }
          else read_file(f);//just read
          fclose(f);
        }
        break;
      }
      case 'a':{
        awaiting=0;
        FILE* f=file_open(line);
        if(f){
          load_file(f);//load into memory
          fclose(f);
        }
        break;
      }
      case 'l':{
        awaiting=0;
        if(lfile)free(lfile);
        lfile=strdup(line);
        break;
      }
      case 'd':{
        awaiting=0;
        delim=strdup(line);
        break;
      }
      case 't':{
        awaiting=0;
        if(lfile){//if we want to apply to another file
            FILE* l=file_open(lfile);
            if(l){
                interlace_to_file(line,l);
                fclose(l);
            }
            free(lfile);
            lfile=0;
        }
        else interlace_line(line);
        break;
      }
      case 'D':{
        awaiting=0;
        free(Fd);
        Fd=strdup(line);
        break;
      }
      case 'F':{
        awaiting=0;
        free(Ff);
        Ff=strdup(line);
        break;
      }
    }
    awaiting=0;
    if(tofree)free(tofree);
    return;
  }

  //empty line?
  if(!*line)return;

  //ordinary line of code
  if(lfile){//applying to a file?
    FILE* l=file_open(lfile);
    if(l){
      apply_line_to_file(line,l);
      fclose(l);
    }
    free(lfile);
    lfile=0;
  }else{//just a line
    char* comment;
    array a=evaluate_line(line,&comment);
    if(comment){
      history.push_back(a);
      print_array(a,stdout,delim,'\n');
    }
  }
}

eval_control::eval_control(){//initialization
#ifdef TIMERAND
  srand(time(0));
#else
  {
    uint seed;
    FILE* f=fopen("/dev/urandom","r");
    fread(&seed,sizeof(uint),1,f);
    srand(seed);
    fclose(f);
  }
#endif
  toggle_print=0;
  mempos=0;
  awaiting=0;
  lfile=0;
  delim=strdup(" ");
#if IVALUE==ILONG
  Fd=strdup("%Ld");
#else
  Fd=strdup("%d");
#endif
#if VALUE==LONG
  Ff=strdup("%Lg");
#else
  Ff=strdup("%g");
#endif
  bufsize=BUF_SIZE;
  buffer=(char*)malloc(bufsize);
}

eval_control::~eval_control(){
  //free buffer
  free(buffer);
  //free other allocated strings
  free(delim);
  free(Fd);
  free(Ff);
}
